@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -93,6 +93,13 @@ int Basic_buffer::characters( const int line ) const throw()
   }
 
 
+bool Basic_buffer::one_character() const throw()
+  {
+  return ( data[0].size() == 1 &&
+           ( lines() == 1 || ( lines() == 2 && data[1].size() == 0 ) ) );
+  }
+
+
 int Basic_buffer::operator[]( const Point & p ) const throw()
   {
   if( p.col < 0 || p.col >= characters( p.line ) ) return -1;
