@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 */
 
 #include <algorithm>
+#include <cstdlib>
 #include <string>
 #include <vector>
 
@@ -156,7 +157,7 @@ bool Block::delete_block() throw()
   if( valid() )
     {
     Buffer & buffer = *_bufferp;
-    _bufferp = 0;			// disable adjust functions
+    _bufferp = 0;			// disable adjust functions on block
     buffer.reset_appendable();
     const bool result = buffer.pdelb( _begin, _end );
     buffer.reset_appendable();
