@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cerrno>
 #include <cstdio>
+#include <cstdlib>
 #include <ctime>
 #include <string>
 #include <vector>
@@ -66,64 +67,79 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b ) throw()
 } // end namespace
 
 
+bool Buffer::Change_history::Record::append_record( const Basic_buffer & buffer,
+                                                    Record & r ) throw()
+  {
+  if( r.atom_vector.size() != 1 ) return false;
+  Atom & a1 = atom_vector.back();
+  Atom & a2 = r.atom_vector.back();
+
+  if( !a1.bbp && !a2.bbp && a1.end == a2.begin )
+    { a1.end = a2.end; return true; }		// consecutive insert
+  if( a1.bbp && a2.bbp )
+    {
+    if( a1.begin == a1.end && a2.begin == a2.end )
+      {
+      if( a1.begin == a2.begin )		// delete at same place
+        {
+        Point p = a1.bbp->eof();
+        a1.bbp->pputb( p, *a2.bbp, a2.bbp->bof(), a2.bbp->eof() );
+        delete a2.bbp; a2.bbp = 0; return true;
+        }
+      if( a2.bbp->one_character() && buffer.pprev( a1.begin ) && a1.begin == a2.begin )
+        {					// consecutive delete
+        Point p = a1.bbp->bof();
+        a1.bbp->pputb( p, *a2.bbp, a2.bbp->bof(), a2.bbp->eof() );
+        a1.end = a1.begin; delete a2.bbp; a2.bbp = 0; return true;
+        }
+      else a1.begin = a1.end;
+      }
+    else if( a1.begin < a1.end && a2.begin < a2.end && a1.end == a2.begin )
+      { add_atom( a2 ); return true; }		// consecutive overwrite
+    }
+  return false;
+  }
+
+
+void Buffer::Change_history::Record::delete_atoms() throw()
+  {
+  for( unsigned int i = 0; i < atom_vector.size(); ++i )
+    if( atom_vector[i].bbp )
+      { delete atom_vector[i].bbp; atom_vector[i].bbp = 0; }
+  }
+
+
 Buffer::Change_history::~Change_history() throw()
   {
   for( unsigned int i = 0; i < record_vector.size(); ++i )
-    {
-    std::vector< Atom > & atom_vector = record_vector[i].atom_vector;
-    for( unsigned int j = 0; j < atom_vector.size(); ++j )
-      if( atom_vector[j].bbp ) delete atom_vector[j].bbp;
-    }
+    record_vector[i].delete_atoms();
   }
 
 
 void Buffer::Change_history::add_record( const Basic_buffer & buffer,
-                                         const Record & r ) throw()
+                                         Record & r ) throw()
   {
   if( index + 1 < records() )
     {
+    for( unsigned int i = index + 1; i < record_vector.size(); ++i )
+      record_vector[i].delete_atoms();
     record_vector.erase( record_vector.begin() + index + 1, record_vector.end() );
     appendable = false;
     }
-  if( force_append && records() )
+  if( !records() ) { appendable = false; force_append = false; }
+  if( ( appendable || force_append ) && record_vector.back().append_record( buffer, r ) )
+    record_vector.back().after = r.after;
+  else if( force_append )
     {
     std::vector< Atom > & av = record_vector.back().atom_vector;
     av.insert( av.end(), r.atom_vector.begin(), r.atom_vector.end() );
+    record_vector.back().after = r.after;
     }
-  else if( !appendable || !append_record( buffer, r ) )
-    { record_vector.push_back( r ); ++index; }
+  else { record_vector.push_back( r ); ++index; }
   appendable = true; force_append = false;
   }
 
 
-bool Buffer::Change_history::append_record( const Basic_buffer & buffer,
-                                            const Record & r ) throw()
-  {
-  if( records() )
-    {
-    Record & last = record_vector.back();
-    if( r.atom_vector.size() == 1 )
-      {
-      Atom & a1 = last.atom_vector.back();
-      const Atom & a2 = r.atom_vector.back();
-      if( !a1.bbp && !a2.bbp && a1.begin <= a2.end && a1.end >= a2.begin )
-        { last.atom_vector.push_back( a2 ); return true; }
-      if( a1.bbp && a2.bbp && a1.begin != a1.end && a2.begin != a2.end &&
-          a1.begin <= a2.end && a1.end >= a2.begin )
-        { last.atom_vector.push_back( a2 ); return true; }
-      if( a1.bbp && a2.bbp && a1.begin == a1.end && a2.begin == a2.end )
-        {
-        Point p1 = a1.begin, p2 = a1.begin;
-        if( a1.begin == a2.begin || ( buffer.pprev( p1 ) && p1 == a2.begin ) ||
-            ( buffer.pnext( p2 ) && p2 == a2.begin ) )
-          { last.atom_vector.push_back( a2 ); return true; }
-        }
-      }
-    }
-  return false;
-  }
-
-
 bool Buffer::Options::set_lmargin( const int l ) throw()
   {
   if( l >= 0 && l < _rmargin ) { _lmargin = l; return true; }
@@ -278,15 +294,16 @@ const std::string & Buffer::uname() const throw()
 
 // insert block b.[p1,p2) at p and move next
 bool Buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1,
-                    const Point & p2, const bool save_change ) throw()
+                    const Point & p2, const int save_change ) throw()
   {
   if( options.read_only || !pisvalid( p ) || !b.pisvalid( p1 ) ||
       !b.pisvalid( p2 ) || p1 >= p2 ) return false;
   const Point p0 = p;
   Basic_buffer::pputb( p, b, p1, p2 );
-  if( save_change )
+  if( save_change )		// save_change == 0 only for undo/redo
     {
-    Change_history::Record r( p0, _modified );
+    const Point & after = (save_change != 2) ? p0 : p;
+    Change_history::Record r( p0, after, _modified );
     r.add_atom( Change_history::Atom( p0, p ) );
     change_history.add_record( *this, r );
     _modified = true;
@@ -323,30 +340,32 @@ bool Buffer::pputc( Point & p, const unsigned char ch ) throw()
       if( pt.col > 0 && p.col >= pt.col )
         tmp.pputb( dummy, *this, bol( pt ), pt );
       }
+    const bool result = pputb( p, tmp, tmp.bof(), tmp.eof(), 2 );
     reset_appendable();
-    return pputb( p, tmp, tmp.bof(), tmp.eof() );
+    return result;
     }
   else if( options.lmargin() > 0 && p.col == 0 && p == eol( p ) )
     {
     for( int i = 0; i < options.lmargin(); ++i ) tmp.pputc( dummy, ' ' );
     tmp.pputc( dummy, ch );
     reset_appendable();
-    return pputb( p, tmp, tmp.bof(), tmp.eof() );
+    return pputb( p, tmp, tmp.bof(), tmp.eof(), 2 );
     }
   else
     {
     const Point p0 = p;
-    Change_history::Record r( p0, _modified );
     if( options.overwrite && p != eol( p ) )
       {
       tmp.pputc( dummy, (*this)[p] );
       Basic_buffer::pchgc( p, ch );
+      Change_history::Record r( p0, p, _modified );
       r.add_atom( Change_history::Atom( p0, p, tmp, tmp.bof(), tmp.eof() ) );
       change_history.add_record( *this, r );
       }
     else
       {
       Basic_buffer::pputc( p, ch );
+      Change_history::Record r( p0, p, _modified );
       r.add_atom( Change_history::Atom( p0, p ) );
       change_history.add_record( *this, r );
       std::vector< Point * > & pv = get_marks_and_pointers( *this );
@@ -368,7 +387,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch ) throw()
           if( !options.auto_indent && options.lmargin() > 0 )
             {
             for( int i = 0; i < options.lmargin(); ++i ) tmp.pputc( dummy, ' ' );
-            force_append(); pputb( p2, tmp, tmp.bof(), tmp.eof() );
+            force_append(); pputb( p2, tmp, tmp.bof(), tmp.eof(), 2 );
             }
           }
         }
@@ -384,10 +403,10 @@ bool Buffer::pdelb( const Point & p1, const Point & p2, const int save_change )
   {
   if( options.read_only || p1 >= p2 || !pisvalid( p1 ) || !pisvalid( p2 ) )
     return false;
-  if( save_change )
+  if( save_change )		// save_change == 0 only for undo/redo
     {
     const Point & p0 = (save_change != 2) ? p1 : p2;
-    Change_history::Record r( p0, _modified );
+    Change_history::Record r( p0, p1, _modified );
     r.add_atom( Change_history::Atom( p1, p2, *this ) );
     change_history.add_record( *this, r );
     _modified = true;
@@ -446,12 +465,12 @@ bool Buffer::undo( Point & p ) throw()
     atom.end = atom.begin;
     if( atom.bbp )
       {
-      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), false );
-      delete( atom.bbp );
+      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), 0 );
+      delete atom.bbp;
       }
     atom.bbp = tmp;
     }
-  p = rec.pointer;
+  p = rec.before;
   std::swap( _modified, rec.modified );
   --change_history.index;
   return true;
@@ -472,17 +491,16 @@ bool Buffer::redo( Point & p ) throw()
       {
       tmp = new Basic_buffer( *this, atom.begin, atom.end );
       pdelb( atom.begin, atom.end, 0 );
-      p = atom.begin;
       }
     atom.end = atom.begin;
     if( atom.bbp )
       {
-      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), false );
-      p = atom.end; delete( atom.bbp );
+      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), 0 );
+      delete atom.bbp;
       }
     atom.bbp = tmp;
     }
-  if( p.line > rec.pointer.line ) p = rec.pointer;
+  p = rec.after;
   std::swap( _modified, rec.modified );
   return true;
   }
@@ -540,23 +558,24 @@ int Bufhandle_vector::handles( const Buffer & buffer ) const throw()
   }
 
 
-int Bufhandle_vector::add_handle( const Buffer::Options & opts,
-                                  const std::string * namep,
-                                  int i, const int line ) throw( Buffer::Error )
+int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
+                                          const std::string * namep,
+                                          int i, const int line ) throw( Buffer::Error )
   {
-  Buffer *bufp = 0;
   if( namep && namep->size() )
-    {
     for( int j = 0; j < handles(); ++j )
       if( *namep == data[j].buffer().name() )
-        { bufp = &data[j].buffer(); break; }
-    }
-  if( !bufp ) bufp = new Buffer( opts, namep );
+        return ( j + handles() - _first ) % handles();
+
+  Buffer *bufp = new Buffer( opts, namep );
+  if( namep && namep->size() ) RC::apply_regex_options( *bufp );
 
   if( i < 0 ) i = handles();
   else
-    { i = ( _first + i ) % handles();
-    if( i == 0 ) i = handles(); else if( i <= _first ) ++_first; }
+    {
+    i = ( _first + i ) % handles();
+    if( i == 0 ) i = handles(); else if( i <= _first ) ++_first;
+    }
   data.insert( data.begin() + i, Buffer_handle( *bufp, line ) );
   return ( i + handles() - _first ) % handles();
   }
@@ -566,7 +585,7 @@ int Bufhandle_vector::add_handle( const Buffer::Options & opts,
 void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line ) throw( Buffer::Error )
   {
   if( !handles() || !isatty( fileno( stdin ) ) )
-    add_handle( RC::default_buffer_options(), 0, -1, line );
+    find_or_add_handle( RC::default_buffer_options(), 0, -1, line );
   }
 
 
