@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -236,6 +236,13 @@ void ISO_8859::deescapize( std::string & s ) throw()
   }
 
 
+void ISO_8859::detabize( std::string & s ) throw()
+  {
+  for( int i = s.size(); --i >= 0; )
+    if( s[i] == '\t' ) { s[i] = 't'; s.insert( i, 1, '\\' ); }
+  }
+
+
 bool ISO_8859::isalpha_( const unsigned char ch ) throw()
   {
   return ( ch == '_' || ISO_8859::islower( ch ) || ISO_8859::isupper( ch ) );
