@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,11 @@ int escape( const std::string & seq, int *lenp = 0 ) throw();
 void deescapize( std::string & s ) throw();
 
 
+// Replace tabs with tab escape sequences; '\t' --> "\t"
+//
+void detabize( std::string & s ) throw();
+
+
 bool isalpha_( const unsigned char ch ) throw();
 bool isalnum_( const unsigned char ch ) throw();
 bool isblank( const unsigned char ch ) throw();
