@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,8 +42,8 @@ const char *help_line[] = {
   " ^[K End of block    ^KY Delete                         ^OC Center line",
   " ^[G Match pair ->   ^KR Read                           ^OR Refresh",
   " ^[H Match pair <-   ^KW Write                          ^OS Split window",
-  " ^[D Center cursor   ^KI Indent                         ^Q@bF2@b Save named buffers",
-  "                     ^KU Unindent",
+  " ^[D Center cursor   ^KI Indent                         ^OV Show version",
+  "                     ^KU Unindent                       ^Q@bF2@b Save named buffers",
   " @bSCROLL@b",
   " ^[W Up",
   " ^[Z Down            @bBOOKMARKS@b",
