@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -451,10 +451,9 @@ int RC::process_options( int argc, char *argv[] ) throw()
         std::fprintf( stderr, "Warning: `%s' is not a regular file\n", name.c_str() );
         name.clear(); non_regular_found = true;
         }
-      const int i = bufhandle_vector.add_handle( _default_buffer_options, &name, -1, line );
+      const int i = bufhandle_vector.find_or_add_handle( _default_buffer_options, &name, -1, line );
       line = 0;
       Buffer & buffer = bufhandle_vector[i].buffer();
-      apply_regex_options( buffer );
       for( ; optind < argc; ++optind )		// Process file options
         {
         if( argv[optind][0] == '+' )
