@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ bool match_regex( const Buffer & buffer, Point & p,
     ch1 = regex[i];
     switch( ch1 )	// ch1 is first (maybe only) of a escape secuence
       {
-      case '^': if( p1 != buffer.bol( p1 ) ) fail = true; break;
+      case '^': if( p1 != buffer.bol( p1 ) || p1 == buffer.eof() ) fail = true; break;
       case '$': if( p1 != buffer.eol( p1 ) ) fail = true; break;
       case '<': if( !buffer.pisbow( p1 ) ) fail = true; break;
       case '>': if( !buffer.piseow( p1 ) ) fail = true; break;
@@ -313,7 +313,6 @@ bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
         if( !buffer.pputb( p2, tmp, tmp.bof(), tmp.eof() ) ) fail = true;
         buffer.reset_appendable();
         }
-      if( !fail && matched.empty() && !buffer.pnext( p2 ) ) fail = true;
       }
     }
   return !fail;
