@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -162,7 +162,7 @@ void control_k_loop( int key ) throw()
   }
 
 
-void control_o_loop( int key ) throw()
+void control_o_loop( int key, const std::string & version_msg ) throw()
   {
   Window & w = window_vector.curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
@@ -174,6 +174,7 @@ void control_o_loop( int key ) throw()
     case 'C': window_vector.center_line(); break;
     case 'R': window_vector.repaint(); break;
     case 'S': window_vector.split_window(); break;
+    case 'V': show_message( version_msg ); break;
     }
   }
 
@@ -243,7 +244,7 @@ bool Screen::init() throw()
 int Screen::wait_kbhit( const int new_clock_limit, const bool get_key ) throw()
   {
   if( new_clock_limit < 0 )
-    { move_to( window_vector.curwin().absolute_cursor() ); }
+    move_to( window_vector.curwin().absolute_cursor() );
 
   clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
   if( clock_limit > 0 ) { clock_handler( 0 ); alarm( 1 ); }
@@ -257,6 +258,16 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key ) throw()
   }
 
 
+int Screen::kbhit() throw()
+  { 
+  nodelay( stdscr, true );
+  int key = wgetch( stdscr );
+  nodelay( stdscr, false );
+  if( key == ERR ) return 0;
+  else { ungetch( key ); return key; }
+  }
+
+
 void Screen::clock_handler( const int sig ) throw()	// updates the clock
   {
   static char c = ':';
@@ -329,8 +340,10 @@ int Screen::get_string( const std::string & prompt,
                           Block::begin().line == Block::end().line )
                         {
                         std::string tmp;
-                        unsigned int len = Block::buffer().to_string( Block::begin(), Block::end(), tmp );
-                        if( len ) { s.insert( scol, tmp ); scol += len; modified = true; }
+                        Block::buffer().to_string( Block::begin(), Block::end(), tmp );
+                        ISO_8859::detabize( tmp );
+                        if( tmp.size() )
+                          { s.insert( scol, tmp ); scol += tmp.size(); modified = true; }
                         }
                       break;
       case KEY_DC   : if( scol < s.size() )
@@ -498,8 +511,9 @@ Screen::Keys Screen::convert_key( const int key ) throw()
   }
 
 
-int Screen::user_loop() throw()
+int Screen::user_loop( const std::string & version_msg ) throw()
   {
+  show_message( version_msg );
   int retval = -1;
   while( retval < 0 )
     {
@@ -508,19 +522,19 @@ int Screen::user_loop() throw()
     Window & w = window_vector.curwin();
     switch( key )
       {
-      case  0: window_vector.toggle_marking(); break;	// ^SPACE
-      case  2: window_vector.reformat(); break;		// ^B
-      case  3: if( !window_vector.close( true ) ) retval = 0; break;  // ^C
-      case  6: window_vector.search(); break;		// ^F
-      case 11: control_k_loop( key ); break;		// ^K
-      case 12: window_vector.search( true ); break;	// ^L
-      case 15: control_o_loop( key ); break;		// ^O
-      case 16: user_control_char(); break;		// ^P
-      case 17: control_q_loop( key, retval ); break;	// ^Q
-      case 19: w.show_character_info(); break;		// ^S
-      case 24: if( !window_vector.close( false ) ) retval = 0; break; // ^X
-      case 25: window_vector.delete_line(); break;	// ^Y
-      case 27: alt_key_loop( key ); break;		// Alt or Esc
+      case  0: window_vector.toggle_marking(); break;		// ^SPACE
+      case  2: window_vector.reformat(); break;			// ^B
+      case  3: if( !window_vector.close( true ) ) retval = 0; break;	// ^C
+      case  6: window_vector.search(); break;			// ^F
+      case 11: control_k_loop( key ); break;			// ^K
+      case 12: window_vector.search( true ); break;		// ^L
+      case 15: control_o_loop( key, version_msg ); break;	// ^O
+      case 16: user_control_char(); break;			// ^P
+      case 17: control_q_loop( key, retval ); break;		// ^Q
+      case 19: w.show_character_info(); break;			// ^S
+      case 24: if( !window_vector.close( false ) ) retval = 0; break;	// ^X
+      case 25: window_vector.delete_line(); break;		// ^Y
+      case 27: alt_key_loop( key ); break;			// Alt or Esc
       case KEY_F(1) : Help::help_menu( key ); break;
       case KEY_F(2) : window_vector.save_file(); break;
       case KEY_F(3) : window_vector.load_file(); break;
