@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ bool init() throw();
 // This function is the real "main loop" of the program.
 //
 int wait_kbhit( const int new_clock_limit = -1, const bool get_key = true ) throw();
+int kbhit() throw();
 
 void clock_handler( const int sig ) throw();		// updates the clock
 
@@ -61,6 +62,6 @@ int show_message( const std::string & s, const bool get_key = false,
 void show_status_lines() throw();
 
 Keys convert_key( const int key ) throw();
-int user_loop() throw();
+int user_loop( const std::string & version_msg ) throw();
 
 } // end namespace Screen
