@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -175,7 +175,7 @@ void Window::goto_matching_delimiter( const bool forward ) throw()
     {
     if( p == buffer.eol( p ) )
       {
-      --p.col;
+      if( --p.col > 0 && buffer[p] == ';' ) --p.col;
       if( buffer.set_to_matching_delimiter( p, forward ) )
         { update_points( p ); return; }
       }
@@ -184,6 +184,10 @@ void Window::goto_matching_delimiter( const bool forward ) throw()
       if( buffer.set_to_matching_delimiter( p, forward ) )
         { update_points( p ); return; }
     }
+  p = buffer.bot( bhp->pointer );
+  if( bhp->pointer < p && p < buffer.eol( p ) &&
+      buffer.set_to_matching_delimiter( p, forward ) )
+    { update_points( p ); return; }
   Screen::show_message( "No matching delimiter" );
   }
 
