@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -115,13 +115,15 @@ void Window_vector::load_file() throw()
   std::string name;
   if( File::get_filename( "Name of file to edit (^C to abort): ", name ) < 0 )
     return;
+  const int old_handles = bufhandle_vector.handles();
   int handle;
-  try { handle = bufhandle_vector.add_handle( RC::default_buffer_options(),
-                                              &name, _curwin + 1 ); }
+  try { handle = bufhandle_vector.find_or_add_handle( RC::default_buffer_options(),
+                                                      &name, _curwin + 1 ); }
   catch( Buffer::Error e ) { Screen::show_message( e.s ); return; }
-  if( name.size() ) RC::apply_regex_options( bufhandle_vector[handle].buffer() );
-  ++_curwin;
-  if( windows() > 1 ) load( windows() + 1 ); else load();
+  _curwin = handle;
+  if( windows() > 1 && bufhandle_vector.handles() > old_handles )
+    load( windows() + 1 );
+  else load();
   }
 
 
@@ -553,6 +555,7 @@ void Window_vector::search( const bool again ) throw()
   {
   static std::vector< std::string > search_history;
   static std::vector< std::string > replace_history;
+  static std::vector< std::string > options_history;
   static std::string search_string;
   static std::string replace_string;
   static bool backward = false, block = false, global = false, icase = false;
@@ -560,25 +563,24 @@ void Window_vector::search( const bool again ) throw()
 
   if( !again || search_history.size() == 0 )
     {
-    search_string.clear();
     search_history.push_back( std::string() );
-    if( Screen::get_string( "Find (^C to abort): ", search_history, true ) <= 0 ) return;
+    if( Screen::get_string( "Find (^C to abort): ", search_history, true ) <= 0 )
+      return;
     search_string = search_history.back(); ISO_8859::deescapize( search_string );
     std::string prompt;
     if( RC::options().ignore_case )
       prompt = "(N)o_ignore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     else prompt = "(I)gnore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
-    std::vector< std::string > mode;
-    mode.push_back( std::string() );
-    const int result = Screen::get_string( prompt, mode );
+    options_history.push_back( std::string() );
+    const int result = Screen::get_string( prompt, options_history );
     if( result < 0 ) return;
     backward = false; block = false; global = false; replace = false;
     icase = RC::options().ignore_case;
     if( result > 0 )
       {
-      std::string & m = mode.back();
-      for( unsigned int i = 0; i < m.size(); ++i )
-        switch( ISO_8859::toupper( m[i] ) )
+      const std::string & options = options_history.back();
+      for( unsigned int i = 0; i < options.size(); ++i )
+        switch( ISO_8859::toupper( options[i] ) )
           {
           case 'B': backward = true; break;
           case 'G': global = true; break;
@@ -606,21 +608,25 @@ void Window_vector::search( const bool again ) throw()
     if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return;
 //    if( RC::options().rectangle_mode )
 //      { Screen::show_message( "Can't search block in rectangle mode" ); return; }
-    if( backward ) { if( p > Block::end() ) p = Block::end(); }
-    else { if( p < Block::begin() ) p = Block::begin(); }
+    if( !again || !Block::in_block_or_end( curwin().buffer(), p ) )
+      { if( backward ) p = Block::end(); else p = Block::begin(); }
     }
   bool found = false, ask = true, found_any = false, wrapped = false;
+  bool empty_replace = false;
   for( int i = (global ? bufhandle_vector.handles() - 1 : 0); i >= 0 && !found; --i )
     {
     Window & w = data[_curwin];
     std::vector< std::string > regex_pieces;
+    if( replace && empty_replace && !backward ) w.buffer().pnext( p );
     Point p1 = p, p2 = p;
     found = Regex::find( w.buffer(), p1, p2, search_string, regex_pieces, icase, backward );
+    empty_replace = ( found && ( p1 == p2 ) );
+    if( found && block && ( p1 < Block::begin() || p1 >= Block::end() || p2 > Block::end() ) )
+      found = false;
     if( backward ) p = p1; else p = p2;
 
     if( found )
       {
-      if( block && ( p1 < Block::begin() || p2 > Block::end() ) ) break;
       found_any = true;
       if( !replace ) w.update_points( p, true, true );
       else
@@ -630,19 +636,26 @@ void Window_vector::search( const bool again ) throw()
         if( ask ) { Block::set_block( w.buffer(), p1, p2 ); w.repaint(); }
         while( true )
           {
-          int key = ask ? Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true ) : 'y';
-          if( key == 3 ) break;
-          if( key == 'n' || key == 'N' ) { found = false; break; }
-          if( key == 'r' || key == 'R' ) { key = 'y'; ask = false; }
-          if( key == 'y' || key == 'Y' )
+          int key = 0;
+          if( ask || Screen::kbhit() )
             {
-            if( Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces ) )
-              {
-              found = false; if( backward ) p = p1; else p = p2;
-              w.update_points( p, false, true );
-              }
+            key = Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true );
+            if( key == 3 ) break;
+            if( ask )
+              { if( key == 'n' || key == 'N' ) { found = false; break; }
+                if( key == 'r' || key == 'R' ) ask = false; }
+            }
+          if( !ask || key == 'y' || key == 'Y' )
+            {
+            Block::restore_block_position();
+            const bool b = Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
+            Block::save_block_position();
+            if( b )
+              { if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
+                found = false; if( backward ) p = p1; else p = p2;
+                w.update_points( p, false, true ); }
             else if( w.buffer().options.read_only ) error = "Read only";
-            else if( p2 < w.buffer().eof() ) error = "Replace error: check syntax";
+            else error = "Replace error: check syntax";
             break;
             }
           }
